from textblob import TextBlob

class normalization:
    '''Normalization class'''

    def __init__(self,df,exclusion=[]):
        self.df=df
        self.originaldf=df #keep in memory original data for direct method call
        self.exclusion= exclusion    
   
    def spellcorr(self,call):
        '''Function uses TextBlob to auto correct spelling errors'''

        if (call!=0): # call from wrapper script
            text=self.df#getting text from the class variable
            val=" ".join([str(TextBlob(x).correct()) for x in text.split(' ')])
            self.df=val
        else:# call from main class method
            text=self.originaldf#getting text from the class variable
            val=" ".join([str(TextBlob(x).correct()) for x in text.split(' ')])
        return val
    def normalizmain(self,spellcorr=True):
        '''Main normalization function to call sub funtions'''

        if spellcorr:
            val=self.spellcorr(2)  # pass anything other than 0 for wrapper call 
        #val=self.df
            return val



from wordcloud import WordCloud as wc
import matplotlib.pyplot as plt
from nltk.probability import FreqDist
from nltk.tokenize import word_tokenize


class WordCloud:
    """
    Create word cloud from text
    """
    wc_obj: wc = None

    def __init__(self, **kwargs):
        self.wc_obj = wc(**kwargs)

    def show(self, text):
        """
        Show word cloud for give text.
        This method will process the text by Splits a long text into words, eliminates the stopwords.
        :param text: Text to plot wordcloud
        :return: None
        """
        wcloud = self.wc_obj.generate_from_text(text)
        plt.imshow(wcloud, interpolation='bilinear')
        plt.axis("off")
        plt.show()


class FrequencyDist:
    """
    Class to plot frequency distribution for a given text
    """

    def __init__(self):
        print("Frequency distribution constructor")

    @staticmethod
    def show(text, min_length=3, nof_large_words=50):
        """
        Show words frequency distribution of give text
        :param text: text to plot frequency distribution
        :param min_length: minimum length of the word to consider
        :param nof_large_words: No. of maximum words to plot on graph
        :return:

        :Notes
        ---------
        Given :param text will be not do any preprocessing including lower case, punctuation and others.
        """
        # Convert text to words array using NLTK word tokenizer
        wt_words = word_tokenize(text)
        # Create frequency distribution for a tokenized text
        frequency_dist = FreqDist(wt_words)
        # Filter the words with given minimum length
        large_words = dict([(k, v) for k, v in frequency_dist.items() if len(k) >= min_length])
        # Create frequency distribution for large words
        frequency_dist = FreqDist(large_words)
        # show distribution map
        frequency_dist.plot(nof_large_words, cumulative=False)



import re
# Noise removal using regular expressions
import nltk

nltk.download('stopwords')
nltk.download('words')
# For removing non-english words
from nltk.corpus import stopwords

stop = set(stopwords.words('english'))
# For removing stop words
from emot.emo_unicode import UNICODE_EMO, EMOTICONS


# For converting emojis to words


class textpreprocessor:

    def __init__(self, text):
        self.text = text

    def remove_noise(self, to_lower=True,
                     hash_remove=True,
                     remove_at=True,
                     remove_alphnum=True,
                     remove_URL=True,
                     remove_punct=True,
                     convert_emoji=True,
                     convert_emoticons=True,
                     remove_non_eng=True,
                     remove_stop_words=True
                     ):

        'Function to remove all noise from the text by default.'
        'The default settings can be turned off,by specifying the required parameter as false'

        if to_lower:
            self.lower()

        if hash_remove:
            self.hash_removal()

        if remove_at:
            self.at_removal()

        if remove_URL:
            self.https_removal()

        if remove_alphnum:
            self.remove_alpnum()

        if remove_punct:
            self.punctuation_removal()

        if convert_emoji:
            self.convert_emojis()

        if convert_emoticons:
            self.convert_emoticons()

        if remove_non_eng:
            self.non_eng()

        if remove_stop_words:
            self.stop_word_removal()

        # to remove extra spaces and lines
        self.text = re.sub("[\s]+", ' ', self.text)
        self.text = re.sub("[\n]+", ' ', self.text)
        return self.text

    def lower(self):
        'Function to convert to lower case'
        self.text = self.text.lower()
        return self

    def hash_removal(self):
        'Function to remove # related words'
        self.text = re.sub(r'#[\w]*', ' ', self.text)
        return self

    def at_removal(self):
        'Function to remove @ related words'
        self.text = re.sub(r'@[\w]*', ' ', self.text)
        return self

    def remove_alpnum(self):
        'Function to remove alphanumeric words'
        self.text = re.sub(r'[^\w]', ' ', self.text)
        return self

    def backslash(self):
        'Function to remove \n '
        self.text = self.text.replace("\\n", " ")
        return self

    def https_removal(self):
        'Function to remove https links '
        self.text = re.sub(r"http\S+", "", self.text)
        return self

    def punctuation_removal(self):
        'Function to remove punctuations '
        self.text = re.sub(r"[^a-zA-Z# ]", "", self.text)
        return self

    def convert_emojis(self):
        'Function to convert emojis to text'
        for emot in UNICODE_EMO:
            self.text = self.text.replace(emot, "_".join(UNICODE_EMO[emot].replace(",", "").replace(":", "").split()))
        return self

    def convert_emoticons(self):
        'Function to convert emoticons to text'
        for emot in EMOTICONS:
            self.text = re.sub(u'(' + emot + ')', "_".join(EMOTICONS[emot].replace(",", "").split()), self.text)
        return self

    def non_eng(self):
        'Function to remove non-english words'
        words = set(nltk.corpus.words.words())
        self.text = " ".join(w for w in nltk.wordpunct_tokenize(self.text) if w.lower() in words or not w.isalpha())
        return self

    def stop_word_removal(self):
        'Function to remove stop words'
        words = set(nltk.corpus.words.words())
        self.text = " ".join(word for word in self.text.split()
                             if not word in stop and '#' not in word.lower())
        return self




from racepretextproc.visualization import visualize
from racepretextproc.normalization import normalization
from racepretextproc.noise_remove import noise_remove
from racepretextproc.lemetstemtoken import lemetstemtoken




from setuptools import find_packages, setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="racepretextproc",
    version="0.0.4",
    author='Saby, Rakesh, Shashi, Gurushanth',
    author_email="author@example.com",
    license="MIT",
    description="Race Text processing library",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/race-ai01-srgs/racepretextproc",
    packages=find_packages(exclude=("tests",)),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[            
          'regex',
          'NLTK',
          'textblob',
          'emot',
          'WordCloud',
          'matplotlib'
    ],
    python_requires='>=3.6'
)



from nltk.stem import PorterStemmer
from nltk.tokenize import word_tokenize
from textblob import Word

class lemetization_stemming_tokenization:


    def __init__(self, text):
        self.text = text

   #function for Stemming, Lemmetization, tokenisation
    def lemet_stem_token(self,
                         do_stem=False,
                         do_lemet=True):
        # Tokenize text into words
        self.words = self.getTokens()

        if do_stem:
            self.doStemming()

        if do_lemet:
            self.doLemmetization()

        return self.text
    
    #function for Stemming
    def doStemming(self):
        pst = PorterStemmer()
        self.text = ""
        # for word in self.words:
        #     self.text += " ".join(st.stem(word))
        self.text = " ".join([pst.stem(word) for word in self.words])
        return self.text

    #function for Lemmetization
    def doLemmetization(self):
        # lemmatizer = WordNetLemmatizer()
        self.text = ""
        # for word in self.words:
        #     self.text += " ".join(lemmatizer.lemmatize(word))

        self.text = " ".join([Word(word).lemmatize() for word in self.words])
        return self.text
    
    #function for tokenisation
    def getTokens(self):
        return word_tokenize(self.text)
